/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "schemedata.h"
#include "progressdialog.h"
#include "imagefilter.h"



bool _schemeData::convert(const _dmc &dmc,const int clSize,int mode)
{
 int edge=1;
 uint max=8;

 switch(mode){
   case 0:break;
   case 1:max+=_image.width()+_image.height();break;
   case 2:max+=_image.width();break;
   default:max+=_image.width()-2*edge;
 }


 progressDialog pb(QObject::trUtf8("Converting picture"),QObject::trUtf8("Cancel"),0,max);
 pb.show();


 pb.addValue(1);


 if((_image.format()==QImage::Format_Indexed8)
      ||(_image.format()==QImage::Format_Mono)
      ||(_image.format()==QImage::Format_MonoLSB))
   _image=_image.convertToFormat(QImage::Format_RGB32,Qt::ThresholdDither);


 pb.addValue(1);


//Применяем фильтры для 32-битного изображения
 if(mode>2){
   _imageFilter imageFilter(edge);
   _image=imageFilter.filter(_image,mode-2,&pb);
 }


 pb.addValue(1);
 if(_image.isNull()){
   clear();
   return false;
 }


//---Находим частоты встречаемости цветов---------------------------------------
//--- Adaptive algorithm -------------------------------------------------------

 QHash<QRgb,int>color;
 for(int i=0;i<_image.width();i++)
   for(int j=0;j<_image.height();j++){
     QRgb cl=_image.pixel(i,j);
     int count=color.value(cl,0);
     color.insert(cl,count+1);
   }


 pb.addValue(1);
 if(pb.wasCanceled()){
   clear();
   return false;
 }


 QHash<QRgb,int>::const_iterator it=color.constBegin();
 while(it!=color.constEnd()){
    _colorCount ccItem;
    ccItem._color=it.key();
    ccItem._count=it.value();
    colorCount.append(ccItem);
    ++it;

    if(pb.wasCanceled()){
      clear();
      return false;
    }
 }


 pb.addValue(1);
 if(pb.wasCanceled()){
   clear();
   return false;
 }

//------------------------------------------------------------------------------


//Располагаем массив встречаемости цветов по возрастанию встречаемости
 qSort(colorCount.begin(),colorCount.end(),qGreater<_colorCount>());


 pb.addValue(1);


 int size=colorCount.size();
 if(clSize<size)size=clSize;



//Реализация выборки цветов из разных цветовых групп для улучшения цвета изображения
//Основная фишка алгоритма: выбор цвета из разных частей массива
//groupCount=15 даёт визуально наилучшее качество преобразования (проверено тестами)
//10 15 13? 17? 101 97 43? 49 51
 int groupCount=15;//Количество цветовых групп, т.е. на столько равных частей разбивается массив colorCount
 int groupSize=int(colorCount.size()/groupCount);//Длина любой части массива
 int groupCurrent=0;//Номер текущей цветовой группы
//------------------------------------------------------------------------------

 int counter=0;
 int i=0;
 while(colorTable.size()<size){
   if(i==colorCount.size())break;

   int ik=groupCurrent*groupSize+i;//выборка цветов из разных цветовых групп
   if(ik>=colorCount.size())ik=i;

   QRgb clDMC=dmc.find(QColor(colorCount.at(ik)._color));//Находим похожий цвет DMC
   //Если нашли подходящий цвет, добавляем его в палитру
   if(clDMC!=0x00000000){
     colorTable.removeAll(clDMC);
     colorTable.append(clDMC);
     counter++;
     groupCurrent++;
     if(groupCurrent==groupCount)groupCurrent=0;
   }
   //-------------------------------------------------------------------------
   i++;
 }


 pb.addValue(1);


 for(int i=0;i<QColorDialog::customCount();i++)
   if(i<colorTable.size())QColorDialog::setCustomColor(i,colorTable.at(i));


 _image=_image.convertToFormat(QImage::Format_Indexed8,colorTable.toVector(),Qt::ThresholdDither|Qt::AvoidDither);

 if(mode==1)
   _image=_imageFilter::median(_image,colorTable.toVector(),&pb);//Применяем медианный фильтр для 8-битного изображения

 if(mode==2)
   _image=_imageFilter::medium(_image,&pb);//Применяем усредняющий фильтр для 8-битного изображения


 pb.addValue(1);
 if((_image.isNull())||(pb.wasCanceled())){
   clear();
   return false;
 }


 pb.close();
 return true;
}

