/*
Copyright 2010-2012 Sergey Levin


This file is part of crosti.

Crosti is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Crosti is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with crosti.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "viewer.h"
#include "info.h"

viewer::viewer(QWidget *parent):QWidget(parent)
{
 ui.setupUi(this);

 setWindowIcon(QIcon(Info::dataPath()+"/system/image/logoapp.png"));

 imageLabel=new QLabel(this);
 imageLabel->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
 imageLabel->setScaledContents(true);

 ui.scrollArea->setBackgroundRole(QPalette::Dark);
 ui.scrollArea->setWidget(imageLabel);
 ui.scaleResetBtn->setIcon(QIcon(Info::dataPath()+"/system/image/zoomreset.png"));

 connect(ui.scaleSB,SIGNAL(valueChanged(int)),this,SLOT(setScale(int)));
 connect(ui.scaleResetBtn,SIGNAL(clicked()),this,SLOT(scaleResetBtnPress()));

}


void viewer::setImage(const QImage &img)
{
 _image=img;
 imageLabel->setPixmap(QPixmap::fromImage(img));
 ui.scaleSB->setValue(100);
 setScale(100);
 emit imageChanged(_image);
}

void viewer::setScale(int factor)
{
 imageLabel->resize((factor*imageLabel->pixmap()->size())/100);

 adjustScrollBar(ui.scrollArea->horizontalScrollBar(),factor/100);
 adjustScrollBar(ui.scrollArea->verticalScrollBar(),factor/100);
}



void viewer::scaleResetBtnPress()
{
 ui.scaleSB->setValue(100);
}


void viewer::adjustScrollBar(QScrollBar *scrollBar,double factor)
{
 scrollBar->setValue(int(factor*scrollBar->value()+((factor-1)*scrollBar->pageStep()/2)));
}
